import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_core/theme.dart';
import 'package:syncfusion_flutter_maps/maps.dart';

import '../data_classes.dart';

/// A class containing thickness data for each subdivision of the map.
class IceThicknessModel {
  IceThicknessModel(this.sub_div_id, this.thickness, this.color, this.savedColor);

  final String sub_div_id;
  final int thickness;
  Color color;
  final Color savedColor;
}

/// ChoroplethMap is a stateful widget that contains a choropleth map.
/// The map is created using the Syncfusion Flutter Maps library and
/// coordinates fetched from the server.
class ChoroplethMap extends StatefulWidget {
  const ChoroplethMap({Key? key,
    required this.relation,
    required this.measurements,
    required this.onSelectionChanged,
  }) : super(key: key);

  final Uint8List relation;
  final List<Measurement> measurements;
  final void Function(int _selectedIndex) onSelectionChanged; // Callback function

  @override
  _ChoroplethMapState createState() => _ChoroplethMapState();
}

class _ChoroplethMapState extends State<ChoroplethMap> {
  int _selectedIndex = -1;
  Color _selectedColor = Colors.grey; // Initialise to gray
  late MapShapeSource _dataSource;
  late final MapZoomPanBehavior _zoomPanBehavior = MapZoomPanBehavior();
  List<SubDiv> _subdivisions = <SubDiv>[];

  @override
  void initState() {
    super.initState();
    // Extract all _subdivisions from list of measurements
    for (Measurement measurement in widget.measurements) {
      for (SubDiv subdivision in measurement.subDivs) {
        _subdivisions.add(subdivision);
      }
    };

    // Initialise data source
    _dataSource = MapShapeSource.memory(
      widget.relation,
      shapeDataField: 'sub_div_id',
      dataCount: _subdivisions.length,
      primaryValueMapper: (int index) =>
      _subdivisions[index].sub_div_id,
      shapeColorValueMapper: (int index) =>
      _subdivisions[index].avgThickness, // NB will later be minThickness, fix in server first
      shapeColorMappers: const [
      MapColorMapper(
          from: 0,
          to: 4,
          color: Color(0xFFff0000),
          text: '{0},{4}'),
      MapColorMapper(
          from: 4,
          to: 6,
          color: Color(0xffff6a00),
          text: '6'),
      MapColorMapper(
          from: 6,
          to: 8,
          color: Color(0xFFb1ff00),
          text: '8'),
      MapColorMapper(
          from: 8,
          to: 400,
          color: Color(0xFF00d6ff),
          text: '>8'),
        ],
    );
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        SfMapsTheme(
          data: SfMapsThemeData( // Coloring of selected shape
            selectionColor: _selectedColor,
            selectionStrokeWidth: 3.5,
            selectionStrokeColor: Colors.blue[600],
          ),
          child: SfMaps(
            layers: [
              MapShapeLayer(
                source: _dataSource,
                zoomPanBehavior: _zoomPanBehavior,
                strokeColor: Colors.blue.shade50,
                strokeWidth: 1,
                // Shape selection
                selectedIndex: _selectedIndex,
                onSelectionChanged: (int index) {  // Shape selection behavior
                  setState(() {
                    _selectedIndex = index;
                    _selectedColor = _subdivisions[index].color;
                  });
                  widget.onSelectionChanged(_selectedIndex);
                },
              ),
            ],
          ),
        ),
      ],
    );
  }
}